package net.pricefx.formulaengine.optimization;

import net.pricefx.domain.DMTable;

import java.util.List;
import java.util.Map;

public interface ModelFormulaContextReadWrite extends ModelFormulaContextReadOnly {
    /**
     * Add a table in the model. If the table already exists, it is replaced by this new table.
     * <p>
     * A {@link Map} respecting the structure of {@link DMTable}, including:
     * <ul>
     * <li>{@code label}: the table label</li>
     * <li>{@code fields}: a {@link List} of {@link Map} respecting the structure of {@link DMField}, including {@code name}, {@code label}, {@code type}, {@code format}, {@code key}, {@code dimension}</li>
     * </ul>
     * <p>
     * Sample code:
     * <pre><code>
     *     model.addTable("RevenueByProduct", [
     *             label: "Revenue By Product",
     *             // the fields must respect DMField structure
     *             fields: [
     *                 [ name:"productId", label:"Product ID", type:"TEXT", key:true],
     *                 [ name:"productGroup", label:"ProductGroup", type:"TEXT", dimension:true],
     *                 [ name:"revenue", label:"Revenue", type:"MONEY"],
     *             ]
     *     ])
     * </code></pre>
     *
     * @param name       The table name to be created in this model
     * @param tableSpecs The definition of the table to be created, see above.
     * @return The created model table
     */
    Object addTable(String name, Map<String, Object> tableSpecs);

    /**
     * Sample code:
     * <pre><code>
     *     def ctx = api.getDatamartContext()
     *     def query = ctx.newQuery(ctx.getDataSource("txDS"))
     *         .select("sku", "P")
     *         .select("SUM(amount)", "R")
     *     def table = model.getTable('MyTable')
     *     def nbRowAdded = model.loadTable(table, query, false)
     * </code></pre>
     *
     * @param table    A table in this model
     * @param source   The query to load into the table
     * @param truncate If the table should be truncated before loading
     * @return The number of row added
     */
    long loadTable(Object table, Object source, boolean truncate);

    /**
     * Sample code:
     * <pre><code>
     *     def ctx = api.getDatamartContext()
     *     def t1 = ctx.newQuery(ctx.getDataSource("txDS"))
     *         .select("sku", "P")
     *         .select("SUM(amount)", "R")
     *     def query = ctx.newSqlQuery()
     *     query.addSource(t1)
     *     query.setQuery('SELECT P, R::numeric as R FROM T1 ORDER BY P')
     *     def table = model.getTable('MyTable')
     *     def nbRowAdded = model.loadTable(table, query, false)
     * </code></pre>
     *
     * @param table    A table in this model
     * @param source   The query to load into the table
     * @param truncate If the table should be truncated before loading
     * @return The number of row added
     */
//    long loadTable(Object table, Object source, boolean truncate);

    /**
     * @param table A table in the model
     */
    void truncateTable(final Object table);

    /**
     * @param table A table in the model
     */
    void dropTable(final Object table);

    /**
     * It is the responsibility of the caller to serialize {@code parameters} however they are required by the image.
     * <p>
     * {@code calculationContext} should be unique per model formula calculation that triggers them.
     * <p>
     * Multiple job trigger calculation in a given formula calculation will run in parallel.
     */
    void startJobTriggerCalculation(String imageName, String imageTag, String parameters, String calculationContext);
}
